@@ -51,6 +51,7 @@ typedef struct
   GFileMonitor   *video_file_monitor;
   GnomeDesktopThumbnailFactory *factory;
   gboolean multiplex_thumbnail_generator;
+  gboolean vertical;
   guint n_items;
   guint idle_id;
   GQueue *thumbnails;
@@ -367,9 +368,16 @@ cheese_thumb_view_monitor_cb (GFileMonitor     *file_monitor,
     case G_FILE_MONITOR_EVENT_DELETED:
       cheese_thumb_view_remove_item (thumb_view, file);
       break;
+
+    case G_FILE_MONITOR_EVENT_MOVED:
+      cheese_thumb_view_remove_item (thumb_view, file);
+      cheese_thumb_view_append_item (thumb_view, other_file);
+      break;
+
     case G_FILE_MONITOR_EVENT_CHANGES_DONE_HINT:
       cheese_thumb_view_append_item (thumb_view, file);
       break;
+
     default:
       break;
   }
@@ -602,7 +610,10 @@ cheese_thumb_view_row_inserted_cb (GtkTreeModel    *tree_model,
   CheeseThumbViewPrivate *priv = CHEESE_THUMB_VIEW_GET_PRIVATE (thumb_view);
 
   priv->n_items++;
-  gtk_widget_set_size_request (GTK_WIDGET (thumb_view), -1, -1);
+  if (!priv->vertical)
+    gtk_icon_view_set_columns (GTK_ICON_VIEW (thumb_view), priv->n_items);
+  else
+    gtk_widget_set_size_request (GTK_WIDGET (thumb_view), -1, -1);
 }
 
 static void
@@ -617,6 +628,8 @@ cheese_thumb_view_row_deleted_cb (GtkTreeModel    *tree_model,
     gtk_widget_set_size_request (GTK_WIDGET (thumb_view),
                                  THUMB_VIEW_MINIMUM_WIDTH,
                                  THUMB_VIEW_MINIMUM_HEIGHT);
+  else if (!priv->vertical)
+    gtk_icon_view_set_columns (GTK_ICON_VIEW (thumb_view), priv->n_items);
 }
 
 static void
@@ -674,7 +687,7 @@ cheese_thumb_view_constructed (GObject *object)
 
   gtk_icon_view_set_pixbuf_column (GTK_ICON_VIEW (thumb_view), 0);
 
-  gtk_icon_view_set_columns (GTK_ICON_VIEW (thumb_view), G_MAXINT);
+  gtk_icon_view_set_columns (GTK_ICON_VIEW (thumb_view), -1);
 
   gtk_icon_view_enable_model_drag_source (GTK_ICON_VIEW (thumb_view), GDK_BUTTON1_MASK,
                                           target_table, G_N_ELEMENTS (target_table),
@@ -685,6 +698,7 @@ cheese_thumb_view_constructed (GObject *object)
                                         THUMBNAIL_BASENAME_URL_COLUMN, GTK_SORT_ASCENDING);
                                         
   cheese_thumb_view_fill (thumb_view);
+  cheese_thumb_view_set_vertical (thumb_view, FALSE);
 }
 
 GtkWidget *
@@ -696,6 +710,18 @@ cheese_thumb_view_new ()
   return GTK_WIDGET (thumb_view);
 }
 
+void
+cheese_thumb_view_set_vertical (CheeseThumbView *thumb_view, gboolean vertical)
+{
+  CheeseThumbViewPrivate *priv = CHEESE_THUMB_VIEW_GET_PRIVATE (thumb_view);
+
+  priv->vertical = vertical;
+  if (!priv->vertical && priv->n_items)
+    gtk_icon_view_set_columns(GTK_ICON_VIEW (thumb_view), priv->n_items);
+  else
+    gtk_icon_view_set_columns(GTK_ICON_VIEW (thumb_view), 1);
+}
+
 void
 cheese_thumb_view_start_monitoring_photo_path (CheeseThumbView *thumb_view, const char *path_photos)
 {
@@ -708,7 +734,7 @@ cheese_thumb_view_start_monitoring_photo_path (CheeseThumbView *thumb_view, cons
 
   /* connect signal to photo path */
   file                     = g_file_new_for_path (path_photos);
-  priv->photo_file_monitor = g_file_monitor_directory (file, 0, NULL, NULL);
+  priv->photo_file_monitor = g_file_monitor_directory (file, G_FILE_MONITOR_SEND_MOVED, NULL, NULL);
   g_signal_connect (priv->photo_file_monitor, "changed", G_CALLBACK (cheese_thumb_view_monitor_cb), thumb_view);
 
   g_object_unref (file);
@@ -727,7 +753,7 @@ cheese_thumb_view_start_monitoring_video_path (CheeseThumbView *thumb_view, cons
 
   /* connect signal to video path */
   file                     = g_file_new_for_path (path_videos);
-  priv->video_file_monitor = g_file_monitor_directory (file, 0, NULL, NULL);
+  priv->video_file_monitor = g_file_monitor_directory (file, G_FILE_MONITOR_SEND_MOVED, NULL, NULL);
   g_signal_connect (priv->video_file_monitor, "changed", G_CALLBACK (cheese_thumb_view_monitor_cb), thumb_view);
 
   g_object_unref (file);
